/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#pragma once

#include <QObject>
#include <QMap>
#include <QHash>
#include <QSet>
#include <QPointer>
#include <interfaces/core/ihookproxy.h>
#include <interfaces/imwproxy.h>

class QMainWindow;
class QDockWidget;
class QAction;

namespace LeechCraft
{
	class MainWindow;
	class RootWindowsManager;
	class DockToolbarManager;

	class DockManager : public QObject
	{
		Q_OBJECT

		RootWindowsManager *RootWM_;

		struct DockInfo
		{
			QWidget *Associated_ = nullptr;
			MainWindow *Window_ = nullptr;
			int Width_ = -1;

			std::optional<QByteArray> SizeContext_ = {};
		};
		QHash<QDockWidget*, DockInfo> Dock2Info_;
		QHash<QAction*, QDockWidget*> ToggleAct2Dock_;
		QSet<QDockWidget*> ForcefullyClosed_;

		QHash<QMainWindow*, DockToolbarManager*> Window2DockToolbarMgr_;
	public:
		DockManager (RootWindowsManager*, QObject* = 0);

		void AddDockWidget (QDockWidget*, const IMWProxy::DockWidgetParams&);
		void AssociateDockWidget (QDockWidget*, QWidget*);

		void ToggleViewActionVisiblity (QDockWidget*, bool);

		void SetDockWidgetVisibility (QDockWidget*, bool);

		QSet<QDockWidget*> GetWindowDocks (const MainWindow*) const;
		void MoveDock (QDockWidget *dock, MainWindow *from, MainWindow *to);

		QSet<QDockWidget*> GetForcefullyClosed () const;
	protected:
		bool eventFilter (QObject*, QEvent*);
	private:
		void SetupDockAction (QDockWidget*);
		void SetupSizing (QDockWidget*, const QByteArray&);
		void HandleDockToggled (QDockWidget*, bool);
	public slots:
		void handleTabMove (int, int, int);
	private slots:
		void handleDockDestroyed ();
		void handleTabChanged (QWidget*);

		void handleWindow (int);
	signals:
		void hookDockWidgetActionVisToggled (LeechCraft::IHookProxy_ptr, QMainWindow*, QDockWidget*, bool);
	};
}
