/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "sopluginloader.h"
#include <QPluginLoader>
#include <QVariantMap>
#include <QJsonObject>
#include <QtDebug>

namespace LeechCraft
{
namespace Loaders
{
	SOPluginLoader::SOPluginLoader (const QString& filename)
	: Loader_ { std::make_shared<QPluginLoader> (filename) }
	{
		Loader_->setLoadHints (QLibrary::ExportExternalSymbolsHint);
	}

	quint64 SOPluginLoader::GetAPILevel ()
	{
		return GetLibAPILevel (Loader_->fileName ());
	}

	bool SOPluginLoader::Load ()
	{
		if (IsLoaded_)
			return true;

		const auto res = Loader_->load ();
		if (!res)
			return false;

		IsLoaded_ = true;
		return true;
	}

	bool SOPluginLoader::Unload ()
	{
		if (!IsLoaded_)
		{
			qWarning () << Q_FUNC_INFO
					<< "trying to unload already loaded instance";
			return true;
		}

		IsLoaded_ = false;
		delete Loader_->instance ();
		Loader_->unload ();
		return true;
	}

	QObject* SOPluginLoader::Instance ()
	{
		return IsLoaded_ ?
				Loader_->instance () :
				nullptr;
	}

	bool SOPluginLoader::IsLoaded () const
	{
		return IsLoaded_;
	}

	QString SOPluginLoader::GetFileName () const
	{
		return Loader_->fileName ();
	}

	QString SOPluginLoader::GetErrorString () const
	{
		return Loader_->errorString ();
	}

	QVariantMap SOPluginLoader::GetManifest () const
	{
		return Loader_->metaData ().toVariantMap () ["MetaData"].toMap ();
	}
}
}
