/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#pragma once

#include <QtPlugin>
#include "audiostructs.h"

template<typename>
class QFuture;

namespace Media
{
	/** @brief Interface for plugins supporting recommended artists.
	 *
	 * If a plugin supports fetching information about recommended
	 * artists, for example, based on user's musical taste, it should
	 * implement this interface.
	 *
	 * @sa ISimilarArtists
	 */
	class Q_DECL_EXPORT IRecommendedArtists
	{
	public:
		virtual ~IRecommendedArtists () {}

		/** @brief Requests the recommended artists.
		 *
		 * This function initiates request for the list of recommended
		 * artists for our user and returns a handle through which the
		 * results of this search could be obtained. The handle owns
		 * itself and deletes itself after results are available — see
		 * its documentation for more details.
		 *
		 * The results of the returned handle will typically have only
		 * SimilarityInfo::SimilarTo_ field set, while
		 * SimilarityInfo::Similarity_ field may be unset. Though if the
		 * latter is set it should be interpreted as some kind of "match
		 * percentage" displaying how interesting an artist can be to our
		 * user.
		 *
		 * Also, IPendingSimilarArtists::GetSourceArtistName() can return
		 * an empty string in this case since it there is no source
		 * artist for which recommendations are fetched.
		 *
		 * @param[in] count The number of recommended artists to fetch.
		 */
		virtual QFuture<SimilarityQueryResult_t> RequestRecommended (int count) = 0;
	};
}

Q_DECLARE_INTERFACE (Media::IRecommendedArtists, "org.LeechCraft.Media.IRecommendedArtists/1.0")
