/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#pragma once

#include <QMap>
#include <QStringList>
#include <QPixmap>
#include <QPointer>
#include <QSystemTrayIcon>
#include <interfaces/structures.h>
#include <interfaces/iactionsexporter.h>
#include "concretehandlerbase.h"
#include "eventdata.h"

class QSystemTrayIcon;

namespace LeechCraft
{
namespace AdvancedNotifications
{
	class VisualNotificationsView;

	class SystemTrayHandler : public ConcreteHandlerBase
	{
		Q_OBJECT

		QMap<QString, QSystemTrayIcon*> Category2Icon_;
		QMap<QString, QAction*> Category2Action_;
		QMap<QString, EventData> Events_;

		QMap<QSystemTrayIcon*, QList<EventData>> EventsForIcon_;
		QMap<QSystemTrayIcon*, VisualNotificationsView*> Icon2NotificationView_;

		QMap<QAction*, QList<EventData>> EventsForAction_;
		QMap<QAction*, VisualNotificationsView*> Action2NotificationView_;
	public:
		SystemTrayHandler ();
		~SystemTrayHandler ();

		NotificationMethod GetHandlerMethod () const;
		void Handle (const Entity&, const NotificationRule&);
	private:
		void PrepareSysTrayIcon (const QString&);
		void PrepareLCTrayAction (const QString&);
		void UpdateMenu (QMenu*, const QString&, const EventData&);
		void RebuildState ();
		template<typename T>
		void UpdateIcon (T iconable, const QString&);
		void UpdateSysTrayIcon (QSystemTrayIcon*);
		void UpdateTrayAction (QAction*);
	private slots:
		void handleActionTriggered ();
		void handleActionTriggered (const QString&, int);
		void dismissNotification ();
		void dismissNotification (const QString&);

		void handleTrayActivated (QSystemTrayIcon::ActivationReason);
		void handleLCAction ();
	signals:
		void gotActions (QList<QAction*>, LeechCraft::ActionsEmbedPlace);
	};
}
}
