/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#pragma once

#include <boost/variant.hpp>
#include <QObject>
#include <QHash>
#include <interfaces/idownload.h>
#include <interfaces/core/icoreproxyfwd.h>
#include "common.h"

namespace LeechCraft::Aggregator
{
	class FeedsErrorManager : public QObject
	{
		Q_OBJECT
	public:
		struct ParseError
		{
			QString Error_;

			bool operator== (const ParseError&) const;
		};
		using Error = boost::variant<IDownload::Error, ParseError>;
	private:
		const ICoreProxy_ptr Proxy_;
		QHash<IDType_t, QList<Error>> Errors_;
	public:
		explicit FeedsErrorManager (const ICoreProxy_ptr&);

		void AddFeedError (IDType_t, const Error&);

		void ClearFeedErrors (IDType_t);

		QList<Error> GetFeedErrors (IDType_t) const;
	signals:
		void gotErrors (IDType_t);
		void clearedErrors (IDType_t);
	};
}
