/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include <QtDebug>
#include "parserfactory.h"
#include "parser.h"
#include "rss20parser.h"
#include "rss10parser.h"
#include "rss091parser.h"
#include "atom10parser.h"
#include "atom03parser.h"

namespace LeechCraft
{
namespace Aggregator
{
	ParserFactory& ParserFactory::Instance ()
	{
		static ParserFactory inst;
		return inst;
	}
	
	void ParserFactory::Register (Parser *parser)
	{
		Parsers_.append (parser);
	}

	void ParserFactory::RegisterDefaultParsers ()
	{
		Register (&RSS20Parser::Instance ());
		Register (&Atom10Parser::Instance ());
		Register (&RSS091Parser::Instance ());
		Register (&Atom03Parser::Instance ());
		Register (&RSS10Parser::Instance ());
	}
	
	Parser* ParserFactory::Return (const QDomDocument& doc) const
	{
		for (auto parser : Parsers_)
			if (parser->CouldParse (doc))
				return parser;
		return nullptr;
	}
}
}

