/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#ifndef PLUGINS_AZOTH_CONSOLEWIDGET_H
#define PLUGINS_AZOTH_CONSOLEWIDGET_H
#include <QWidget>
#include <QPointer>
#include <interfaces/ihavetabs.h>
#include "interfaces/azoth/ihaveconsole.h"
#include "ui_consolewidget.h"

namespace LeechCraft
{
namespace Azoth
{
	class IAccount;
	class IHaveConsole;

	class ConsoleWidget : public QWidget
						, public ITabWidget
	{
		Q_OBJECT
		Q_INTERFACES (ITabWidget)

		Ui::ConsoleWidget Ui_;
		QObject *ParentMultiTabs_ = nullptr;
		TabClassInfo TabClass_;

		QPointer<QObject> AsObject_;
		IAccount *AsAccount_;
		IHaveConsole *AsConsole_;
		const IHaveConsole::PacketFormat Format_;
	public:
		ConsoleWidget (QObject*, QWidget* = 0);

		TabClassInfo GetTabClassInfo () const;
		QObject* ParentMultiTabs ();
		void Remove ();
		QToolBar* GetToolBar () const;

		void SetParentMultiTabs (QObject*);
		QString GetTitle () const;
	private slots:
		void handleConsolePacket (QByteArray, IHaveConsole::PacketDirection, const QString&);
		void on_ClearButton__released ();
		void on_EnabledBox__toggled (bool);
	signals:
		void removeTab (QWidget*);
	};
}
}

#endif
