/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "shortcutsmanager.h"
#include <QTextEdit>
#include <QtDebug>
#include "abbrevsmanager.h"

namespace LeechCraft::Azoth::Abbrev
{
	ShortcutsManager::ShortcutsManager (AbbrevsManager *abbrevs, QObject *parent)
	: QObject { parent }
	, Abbrevs_ { abbrevs }
	{
	}

	void ShortcutsManager::HandleTab (QWidget *tab)
	{
		const auto shortcut = new QShortcut { tab };
		shortcut->setKey (Sequence_);
		connect (shortcut,
				&QShortcut::activated,
				this,
				[this, tab] { HandleActivated (tab); });
		Tab2SC_ [tab] = shortcut;

		connect (tab,
				&QObject::destroyed,
				this,
				[this, tab] { Tab2SC_.remove (tab); });
	}

	QMap<QString, ActionInfo> ShortcutsManager::GetActionInfo () const
	{
		return
		{
			{
				"org.LeechCraft.Azoth.Abbrev.Expand",
				{
					tr ("Expand abbreviations in current message edit text."),
					QKeySequence {},
					{}
				}
			},
		};
	}

	void ShortcutsManager::SetShortcut (const QString&, const QKeySequences_t& seqs)
	{
		Sequence_ = seqs.value (0);

		for (const auto sc : Tab2SC_)
			sc->setKey (Sequence_);
	}

	void ShortcutsManager::HandleActivated (QWidget *tab)
	{
		QTextEdit *edit = nullptr;
		QMetaObject::invokeMethod (tab,
				"getMsgEdit",
				Q_RETURN_ARG (QTextEdit*, edit));
		if (!edit)
		{
			qWarning () << Q_FUNC_INFO
					<< "unable to get message edit"
					<< tab;
			return;
		}

		const auto& text = edit->toPlainText ();
		const auto& processed = Abbrevs_->Process (text);
		if (text == processed)
			return;

		edit->setPlainText (processed);
		edit->moveCursor (QTextCursor::EndOfBlock, QTextCursor::MoveAnchor);
	}
}
