/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#pragma once

#include <unordered_map>
#include <QObject>
#include <QSet>
#include <util/sll/util.h>
#include <interfaces/azoth/iclentry.h>
#include <interfaces/azoth/imucentry.h>
#include "structures.h"
#include "entrybase.h"

namespace LeechCraft
{
namespace Azoth
{
namespace Murm
{
	class VkAccount;
	class VkEntry;

	class VkChatEntry : public EntryBase
					  , public IMUCEntry
	{
		Q_OBJECT
		Q_INTERFACES (LeechCraft::Azoth::IMUCEntry)

		ChatInfo Info_;
		QSet<int> PendingUserInfoRequests_;

		std::unordered_map<VkEntry*, Util::DefaultScopeGuard> EntriesGuards_;
	public:
		enum class HandleMessageResult
		{
			Accepted,
			Rejected,
			UserInfoRequested
		};

		VkChatEntry (const ChatInfo&, VkAccount*);

		void Send (VkMessage*) override;
		HandleMessageResult HandleMessage (const MessageInfo&, const FullMessageInfo&);

		const ChatInfo& GetInfo () const;
		void UpdateInfo (const ChatInfo&);

		void HandleAdded (const UserInfo&);
		void HandleRemoved (qulonglong);

		Features GetEntryFeatures () const override;
		EntryType GetEntryType () const override;
		QString GetEntryName () const override;
		void SetEntryName (const QString& name) override;
		QString GetEntryID () const override;
		QString GetHumanReadableID () const override;
		QStringList Groups () const override;
		void SetGroups (const QStringList& groups) override;
		QStringList Variants () const override;
		void SetChatPartState (ChatPartState state, const QString& variant) override;
		EntryStatus GetStatus (const QString& variant = QString ()) const override;
		void ShowInfo () override;
		QList<QAction*> GetActions () const override;
		QMap<QString, QVariant> GetClientInfo (const QString&) const override;
		void ChatTabClosed () override;

		MUCFeatures GetMUCFeatures () const override;
		QString GetMUCSubject () const override;
		bool CanChangeSubject () const override;
		void SetMUCSubject (const QString& subject) override;
		QList<QObject*> GetParticipants () override;
		bool IsAutojoined () const override;
		void Join () override;
		void Leave (const QString&) override;
		QString GetNick () const override;
		void SetNick (const QString&) override;
		QString GetGroupName () const override;
		QString GetRealID (QObject*) const override;
		QVariantMap GetIdentifyingData () const override;
		void InviteToMUC (const QString& userId, const QString& msg) override;
	private slots:
		void handleGotUsers (const QList<UserInfo>&);
	signals:
		void gotNewParticipants (const QList<QObject*>& parts) override;
		void mucSubjectChanged (const QString& newSubj) override;
		void nicknameConflict (const QString& usedNick) override;
		void beenKicked (const QString& reason) override;
		void beenBanned (const QString& reason) override;

		void removeEntry ();
	};
}
}
}
