/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "sarin.h"
#include <QIcon>
#include <interfaces/azoth/iclentry.h>
#include <tox/tox.h>
#include "toxprotocol.h"

namespace LeechCraft
{
namespace Azoth
{
namespace Sarin
{
	void Plugin::Init (ICoreProxy_ptr proxy)
	{
		qRegisterMetaType<EntryStatus> ("EntryStatus");

		Proto_ = std::make_shared<ToxProtocol> (proxy, this);
	}

	void Plugin::SecondInit ()
	{
	}

	QByteArray Plugin::GetUniqueID () const
	{
		return "org.LeechCraft.Azoth.Sarin";
	}

	void Plugin::Release ()
	{
		Proto_.reset ();
	}

	QString Plugin::GetName () const
	{
		return "Azoth Sarin";
	}

	QString Plugin::GetInfo () const
	{
		return tr ("Tox protocol support for Azoth.");
	}

	QIcon Plugin::GetIcon () const
	{
		return {};
	}

	QString Plugin::GetDiagInfoString () const
	{
		return QString { "Built with Tox %1.%2.%3, running with Tox %4.%5.%6" }
				.arg (TOX_VERSION_MAJOR)
				.arg (TOX_VERSION_MINOR)
				.arg (TOX_VERSION_PATCH)
				.arg (tox_version_major ())
				.arg (tox_version_minor ())
				.arg (tox_version_patch ());
	}

	QSet<QByteArray> Plugin::GetPluginClasses () const
	{
		return { "org.LeechCraft.Plugins.Azoth.Plugins.IProtocolPlugin" };
	}

	QObject* Plugin::GetQObject ()
	{
		return this;
	}

	QList<QObject*> Plugin::GetProtocols () const
	{
		return { Proto_.get () };
	}
}
}
}

LC_EXPORT_PLUGIN (leechcraft_azoth_sarin, LeechCraft::Azoth::Sarin::Plugin);
