/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "util.h"
#include <QByteArray>
#include <QtDebug>
#include <tox/tox.h>

namespace LeechCraft
{
namespace Azoth
{
namespace Sarin
{
	boost::optional<qint32> GetFriendId (const Tox *tox, const QByteArray& pubkey)
	{
		const auto& binPkey = QByteArray::fromHex (pubkey);
		TOX_ERR_FRIEND_BY_PUBLIC_KEY error {};

		const auto res = tox_friend_by_public_key (tox, reinterpret_cast<const uint8_t*> (binPkey.constData ()), &error);
		if (res == UINT32_MAX)
		{
			qWarning () << Q_FUNC_INFO
					<< "failed to get friend by public key"
					<< pubkey
					<< error;
			return {};
		}

		return res;
	}

	QByteArray GetFriendId (const Tox *tox, int32_t friendId)
	{
		std::array<uint8_t, TOX_PUBLIC_KEY_SIZE> clientId;
		TOX_ERR_FRIEND_GET_PUBLIC_KEY error {};

		if (!tox_friend_get_public_key (tox, friendId, clientId.data (), &error))
		{
			qWarning () << Q_FUNC_INFO
					<< "failed to get friend's public key"
					<< friendId
					<< error;
			throw std::runtime_error { "Cannot get friend's pubkey." };
		}

		return ToxId2HR (clientId);
	}
}
}
}
