/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#pragma once

#include <QObject>
#include <interfaces/azoth/iprotocol.h>
#include <interfaces/core/icoreproxy.h>

namespace LeechCraft
{
namespace Azoth
{
class IProxyObject;

namespace Vader
{
	class MRIMAccount;

	class MRIMProtocol final : public QObject
							 , public IProtocol
	{
		Q_OBJECT
		Q_INTERFACES (LeechCraft::Azoth::IProtocol)

		IProxyObject * const AzothProxy_;
		const ICoreProxy_ptr CoreProxy_;

		QList<MRIMAccount*> Accounts_;
	public:
		MRIMProtocol (IProxyObject*, const ICoreProxy_ptr&, QObject* = nullptr);
		~MRIMProtocol ();

		IProxyObject* GetAzothProxy () const;
		const ICoreProxy_ptr& GetCoreProxy () const;

		QObject* GetQObject () override;
		ProtocolFeatures GetFeatures () const override;
		QList<QObject*> GetRegisteredAccounts () override;
		QObject* GetParentProtocolPlugin () const override;
		QString GetProtocolName () const override;
		QIcon GetProtocolIcon () const override;
		QByteArray GetProtocolID () const override;
		QList<QWidget*> GetAccountRegistrationWidgets (AccountAddOptions) override;
		void RegisterAccount (const QString&, const QList<QWidget*>&) override;
		void RemoveAccount (QObject*) override;
	private:
		void RestoreAccounts ();
	private slots:
		void saveAccounts ();
	signals:
		void accountAdded (QObject*) override;
		void accountRemoved (QObject*) override;
	};
}
}
}
