/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "captchamanager.h"
#include <algorithm>
#include <memory>
#include <QDialog>
#include <QDialogButtonBox>
#include <QVBoxLayout>
#include <QtDebug>
#include "xeps/xmppcaptchamanager.h"
#include "formbuilder.h"
#include "xeps/xmppbobmanager.h"

namespace LeechCraft
{
namespace Azoth
{
namespace Xoox
{
	CaptchaManager::CaptchaManager (XMPPCaptchaManager& captchaMgr, XMPPBobManager& bobMgr, QObject *parent)
	: QObject (parent)
	, CaptchaManager_ (captchaMgr)
	, BobManager_ (bobMgr)
	{
		connect (&captchaMgr,
				&XMPPCaptchaManager::captchaFormReceived,
				this,
				&CaptchaManager::HandleCaptchaReceived);
	}

	void CaptchaManager::HandleCaptchaReceived (const QString& jid, const QXmppDataForm& dataForm)
	{
		auto builder = std::make_shared<FormBuilder> (jid, &BobManager_);

		auto dialog = new QDialog ();
		auto widget = builder->CreateForm (dataForm, dialog);
		dialog->setWindowTitle (widget->windowTitle ().isEmpty () ?
				tr ("Enter CAPTCHA") :
				widget->windowTitle ());
		dialog->setLayout (new QVBoxLayout ());
		dialog->layout ()->addWidget (widget);
		const auto box = new QDialogButtonBox (QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
		dialog->layout ()->addWidget (box);
		dialog->setAttribute (Qt::WA_DeleteOnClose);

		connect (box,
				&QDialogButtonBox::accepted,
				dialog,
				&QDialog::accept);
		connect (box,
				&QDialogButtonBox::rejected,
				dialog,
				&QDialog::reject);

		connect (dialog,
				&QDialog::finished,
				this,
				[this, builder, jid] (int result)
				{
					if (result == QDialog::Accepted)
						CaptchaManager_.SendResponse (jid, builder->GetForm ());
				});

		dialog->show ();
	}
}
}
}
