/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "transfermanager.h"
#include <QXmppTransferManager.h>
#include "accountsettingsholder.h"
#include "clientconnection.h"
#include "glooxaccount.h"
#include "glooxclentry.h"
#include "serverinfostorage.h"
#include "transferjob.h"

namespace LeechCraft
{
namespace Azoth
{
namespace Xoox
{
	TransferManager::TransferManager (QXmppTransferManager& mgr, ClientConnection& conn, GlooxAccount& account)
	: Conn_ { conn }
	, Manager_ { mgr }
	, Account_ { account }
	{
		connect (&Manager_,
				&QXmppTransferManager::fileReceived,
				this,
				[this] (QXmppTransferJob *job)
				{
					if (!Conn_.GetCLEntry (job->jid ()))
						Conn_.CreateEntry (job->jid ());
					emit fileOffered (new TransferJob (job, this));
				});

		auto settings = Account_.GetSettings ();

		auto handleDetectedBSProxy = [settings, this] (const QString& proxy)
		{
			if (settings->GetUseSOCKS5Proxy () && !settings->GetSOCKS5Proxy ().isEmpty ())
				return;

			Manager_.setProxy (proxy);
		};

		auto serverInfoStorage = conn.GetServerInfoStorage ();
		connect (serverInfoStorage,
				&ServerInfoStorage::bytestreamsProxyChanged,
				this,
				handleDetectedBSProxy);

		auto updateFTSettings = [=]
		{
			Manager_.setSupportedMethods (settings->GetFTMethods ());
			Manager_.setProxy (settings->GetUseSOCKS5Proxy () ? settings->GetSOCKS5Proxy () : QString {});

			handleDetectedBSProxy (serverInfoStorage->GetBytestreamsProxy ());
		};
		connect (settings,
				&AccountSettingsHolder::fileTransferSettingsChanged,
				this,
				updateFTSettings);
		updateFTSettings ();
	}

	bool TransferManager::IsAvailable () const
	{
		const auto settings = Account_.GetSettings ();
		return settings->GetFTMethods () != QXmppTransferJob::NoMethod;
	}

	QObject* TransferManager::SendFile (const QString& id,
			const QString& sourceVar, const QString& name, const QString& comment)
	{
		QString target = GlooxCLEntry::JIDFromID (&Account_, id);
		QString var = sourceVar;
		if (var.isEmpty ())
		{
			QObject *entryObj = Conn_.GetCLEntry (target, QString ());
			ICLEntry *entry = qobject_cast<ICLEntry*> (entryObj);
			if (!entry)
				return nullptr;
			var = entry->Variants ().value (0);
		}
		if (!var.isEmpty ())
			target += '/' + var;
		return new TransferJob (Manager_.sendFile (target, name, comment), this);
	}

	GlooxAccount* TransferManager::GetAccount () const
	{
		return &Account_;
	}
}
}
}
