/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#pragma once

#include <QObject>
#include <QMap>
#include <libtorrent/version.hpp>
#include <libtorrent/torrent_handle.hpp>

#if LIBTORRENT_VERSION_NUM >= 10100
#include <libtorrent/torrent_status.hpp>
#endif

namespace LeechCraft
{
namespace BitTorrent
{
	class CachedStatusKeeper : public QObject
	{
	public:
#if LIBTORRENT_VERSION_NUM >= 10200
		using FlagsType_t = libtorrent::status_flags_t;
		constexpr static FlagsType_t AllFlags = FlagsType_t::all ();
#else
		using FlagsType_t = uint32_t;
		constexpr static FlagsType_t AllFlags = 0xffffffff;
#endif
	private:
		struct CachedItem
		{
			libtorrent::torrent_status Status_;
			FlagsType_t ReqFlags_;
		};

		QMap<libtorrent::torrent_handle, CachedItem> Handle2Status_;
	public:
		using QObject::QObject;

		libtorrent::torrent_status GetStatus (const libtorrent::torrent_handle&, FlagsType_t flags = {});
		void HandleStatusUpdatePosted (const libtorrent::torrent_status&);
	};
}
}
