/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2015  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "movetorrentfiles.h"
#include <QFileDialog>
#include <xmlsettingsmanager.h>

namespace LeechCraft
{
namespace BitTorrent
{
	MoveTorrentFiles::MoveTorrentFiles (QStringList oldDirectories, QWidget *parent)
	: QDialog { parent }
	{
		Ui_.setupUi (this);

		Q_ASSERT (!oldDirectories.empty ());

		oldDirectories.removeDuplicates ();

		if (1 == oldDirectories.size ())
			Ui_.OldLocation_->setText (oldDirectories.front ());
		else
		{
			Ui_.OldLocation_->setToolTip (oldDirectories.join (", "));
			Ui_.OldLocation_->setPlaceholderText (tr ("Multiple directories"));
		}

		const auto xsm = XmlSettingsManager::Instance ();
		const auto& moveDirectory = xsm->Property ("LastMoveDirectory",
					xsm->property ("LastSaveDirectory").toString ()).toString ();

		Ui_.NewLocation_->setText (moveDirectory);
	}

	QString MoveTorrentFiles::GetNewLocation () const
	{
		return Ui_.NewLocation_->text ();
	}

	void MoveTorrentFiles::on_Browse__released ()
	{
		const auto& dir = QFileDialog::getExistingDirectory (this, tr ("New location"),
				Ui_.NewLocation_->text ());
		if (dir.isEmpty () || dir == Ui_.NewLocation_->text ())
			return;
		Ui_.NewLocation_->setText (dir);
	}
}
}
