/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2010-2012  Oleg Linkin
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#pragma once

#include <QUrl>
#include <QMap>
#include <QStringList>
#include <QVariant>
#include "ljfriendentry.h"

namespace LeechCraft
{
namespace Blogique
{
namespace Metida
{
	struct LJFriendGroup
	{
		bool Public_;
		QString Name_;
		uint Id_ =  0;
		uint SortOrder_ =  0;
		uint RealId_ =  0;
	};

	struct LJMood
	{
		qint64 Parent_ =  0;
		qint64 Id_ =  0;
		QString Name_;
	};

	struct LJProfileData
	{
		QUrl AvatarUrl_;
		qint64 UserId_ =  0;
		qint64 Caps_ =  0;
		QList<LJFriendGroup> FriendGroups_;
		QList<LJMood> Moods_;
		QStringList Communities_;
		QString FullName_;
		QList<LJFriendEntry_ptr> Friends_;
		QStringList AvatarsID_;
		QList<QUrl> AvatarsUrls_;
		QHash<QString, int> Tags_;
	};

	enum class CommentState
	{
		Frozen,
		Secure,
		Active,
		Deleted
	};

	struct LJCommentEntry
	{
		int NodeId_ =  -1;
		QString Subject_;
		int PosterId_ =  -1;
		CommentState State_ = CommentState::Active;
		int ReplyId_ =  -1;
		int ParentReplyId_ =  -1;
		QString PosterName_;
		QString Text_;
		QDateTime PostingDate_;
		QString NodeSubject_;
		QUrl NodeUrl_;
	};

	namespace LJParserTypes
	{
		class LJParseProfileEntry
		{
			QString Name_;
			QVariantList ValueList_;

		public:
			LJParseProfileEntry ();
			LJParseProfileEntry (const QString& name,
					const QVariantList& value);
			QString Name () const;
			QVariantList Value () const;

			bool ValueToBool () const;
			QString ValueToString () const;
			qint64 ValueToLongLong () const;
			int ValueToInt () const;
			QUrl ValueToUrl () const;
		};
	}

	QDataStream& operator<< (QDataStream& out, const LJFriendGroup& group);
	QDataStream& operator>> (QDataStream& in, LJFriendGroup& group);
	QDataStream& operator<< (QDataStream& out, const LJMood& mood);
	QDataStream& operator>> (QDataStream& in, LJMood& mood);
	QDataStream& operator<< (QDataStream& out, const LJProfileData& data);
	QDataStream& operator>> (QDataStream& in, LJProfileData& data);
}
}
}

Q_DECLARE_METATYPE (LeechCraft::Blogique::Metida::LJParserTypes::LJParseProfileEntry)
Q_DECLARE_METATYPE (LeechCraft::Blogique::Metida::LJProfileData)
