/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#pragma once

#include <QObject>
#include <QAction>
#include <interfaces/iinfo.h>
#include <interfaces/iactionsexporter.h>
#include <interfaces/iquarkcomponentprovider.h>
#include <interfaces/ihaveshortcuts.h>
#include <util/xdg/xdgfwd.h>

namespace LeechCraft
{
namespace Util
{
	class ShortcutManager;
}

namespace Launchy
{
	class FavoritesManager;
	class RecentManager;

	class Plugin : public QObject
				 , public IInfo
				 , public IActionsExporter
				 , public IHaveShortcuts
				 , public IQuarkComponentProvider
	{
		Q_OBJECT
		Q_INTERFACES (IInfo IActionsExporter IHaveShortcuts IQuarkComponentProvider)

		LC_PLUGIN_METADATA ("org.LeechCraft.Launchy")

		ICoreProxy_ptr Proxy_;
		Util::XDG::ItemsFinder *Finder_;
		FavoritesManager *FavManager_;
		RecentManager *RecentManager_;

		Util::ShortcutManager *ShortcutMgr_;
		QAction *FSLauncher_;

		QuarkComponent_ptr LaunchQuark_;
	public:
		void Init (ICoreProxy_ptr) override;
		void SecondInit () override;
		QByteArray GetUniqueID () const override;
		void Release () override;
		QString GetName () const override;
		QString GetInfo () const override;
		QIcon GetIcon () const override;

		QList<QAction*> GetActions (ActionsEmbedPlace) const override;

		QMap<QString, ActionInfo> GetActionInfo () const override;
		void SetShortcut (const QString& id, const QKeySequences_t& sequences) override;

		QuarkComponents_t GetComponents () const override;
	private slots:
		void handleFSRequested ();
	signals:
		void gotActions (QList<QAction*>, LeechCraft::ActionsEmbedPlace) override;
	};
}
}
