/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "connectorbase.h"
#include <algorithm>
#include <QDBusConnectionInterface>
#include <QDBusInterface>
#include <QtDebug>

namespace LeechCraft
{
namespace Liznoo
{
	ConnectorBase::ConnectorBase (const QString& service,
			const QByteArray& context, QObject* parent)
	: QObject { parent }
	, SB_ { QDBusConnection::connectToBus (QDBusConnection::SystemBus,
				"LeechCraft.Liznoo." + context + ".Connector") }
	, Service_ { service }
	{
	}

	bool ConnectorBase::TryAutostart ()
	{
		auto iface = SB_.interface ();
		auto checkRunning = [&iface, this]
		{
			return !iface->registeredServiceNames ()
					.value ().filter (Service_).isEmpty ();
		};
		if (checkRunning ())
			return true;

		iface->startService (Service_);
		if (checkRunning ())
			return true;

		qWarning () << Q_FUNC_INFO
				<< "failed to autostart"
				<< Service_;
		return false;
	}

	bool ConnectorBase::CheckSignals (const QString& path, const QStringList& signalsList)
	{
		const auto& introspect = QDBusInterface
		{
			Service_,
			path,
			"org.freedesktop.DBus.Introspectable",
			SB_
		}.call ("Introspect").arguments ().value (0).toString ();
		return std::all_of (signalsList.begin (), signalsList.end (),
				[&introspect] (const QString& signal) { return introspect.contains (signal); });
	}

	bool ConnectorBase::ArePowerEventsAvailable () const
	{
		return PowerEventsAvailable_;
	}

}
}
