/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "logindconnector.h"
#include <QDBusInterface>
#include <QtDebug>

#include <unistd.h>

namespace LeechCraft
{
namespace Liznoo
{
namespace Logind
{
	LogindConnector::LogindConnector (QObject *parent)
	: QObject { parent }
	{
		QDBusInterface iface
		{
			"org.freedesktop.login1",
			"/org/freedesktop/login1",
			"org.freedesktop.DBus.Introspectable",
			SB_
		};
		if (!iface.isValid ())
			return;

		const auto& introspect = iface.call ("Introspect").arguments ().value (0).toString ();
		if (!introspect.contains ("\"PreparingForSleep\""))
		{
			qWarning () << Q_FUNC_INFO
					<< "no PreparingForSleep for logind";
			return;
		}

		SB_.connect ("org.freedesktop.login1",
				"/org/freedesktop/login1",
				"org.freedesktop.login1.Manager",
				"PreparingForSleep",
				this,
				SLOT (handlePreparing (bool)));
		PowerEventsAvailable_ = true;
	}

	bool LogindConnector::ArePowerEventsAvailable () const
	{
		return PowerEventsAvailable_;
	}

	void LogindConnector::Inhibit ()
	{
		const auto& value = QDBusInterface
		{
			"org.freedesktop.login1",
			"/org/freedesktop/login1",
			"org.freedesktop.login1.Manager"
		}.call ("Inhibit",
				"shutdown:sleep",
				"LeechCraft",
				tr ("Preparing LeechCraft for going to sleep..."),
				"delay");
		qDebug () << value;

		const auto fdVar = value.arguments ().value (0);
		qDebug () << fdVar;

		if (fdVar.canConvert<int> ())
			close (fdVar.toInt ());
	}

	void LogindConnector::handlePreparing (bool goingDown)
	{
		if (goingDown)
		{
			Inhibit ();
			emit gonnaSleep (5000);
		}
		else
			emit wokeUp ();
	}
}
}
}
