/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "ppl.h"
#include <QIcon>
#include <QAction>
#include <QFileDialog>
#include <util/util.h>
#include <interfaces/core/icoreproxy.h>
#include <interfaces/lmp/ilmpproxy.h>
#include "loghandler.h"

namespace LeechCraft
{
namespace LMP
{
namespace PPL
{
	void Plugin::Init (ICoreProxy_ptr proxy)
	{
		Proxy_ = proxy;

		Util::InstallTranslator ("lmp_ppl");

		ActionSync_ = new QAction { tr ("Sync scrobbling log"), this };
		connect (ActionSync_,
				&QAction::triggered,
				this,
				[this]
				{
					QFileDialog dia
					{
						nullptr,
						tr ("Select .scrobbler.log"),
						QDir::homePath (),
						tr ("Scrobbler log (*.scrobbler.log)")
					};
					dia.setFilter (QDir::AllEntries | QDir::AllDirs | QDir::Hidden | QDir::NoDotAndDotDot);
					dia.setAcceptMode (QFileDialog::AcceptOpen);
					if (dia.exec () != QDialog::Accepted)
						return;

					const auto& path = dia.selectedFiles ().value (0);
					if (path.isEmpty ())
						return;

					new LogHandler { path, LMPProxy_->GetLocalCollection (), Proxy_->GetPluginsManager (), this };
				});
	}

	void Plugin::SecondInit ()
	{
	}

	void Plugin::Release ()
	{
	}

	QByteArray Plugin::GetUniqueID () const
	{
		return "org.LeechCraft.LMP.PPL";
	}

	QString Plugin::GetName () const
	{
		return "LMP PPL";
	}

	QString Plugin::GetInfo () const
	{
		return tr ("Synchronizes the logs of songs played on portable devices with LMP and scrobbling services like Last.FM.");
	}

	QIcon Plugin::GetIcon () const
	{
		return {};
	}

	QSet<QByteArray> Plugin::GetPluginClasses () const
	{
		return { "org.LeechCraft.LMP.General" };
	}

	QList<QAction*> Plugin::GetActions (ActionsEmbedPlace) const
	{
		return {};
	}

	QMap<QString, QList<QAction*>> Plugin::GetMenuActions () const
	{
		return
		{
			{
				"LMP",
				{ ActionSync_ }
			}
		};
	}

	void Plugin::SetLMPProxy (ILMPProxy_ptr proxy)
	{
		LMPProxy_ = proxy;
	}
}
}
}

LC_EXPORT_PLUGIN (leechcraft_lmp_ppl, LeechCraft::LMP::PPL::Plugin);
