/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "smoothscroller.h"
#include <QTimeLine>
#include "pagesview.h"
#include "xmlsettingsmanager.h"

namespace LeechCraft
{
namespace Monocle
{
	SmoothScroller::SmoothScroller (PagesView *view, QObject *parent)
	: QObject { parent }
	, View_ { view }
	, ScrollTimeline_ { new QTimeLine { 400, this } }
	{
		ScrollTimeline_->setFrameRange (0, 100);
		connect (ScrollTimeline_,
				&QTimeLine::frameChanged,
				this,
				&SmoothScroller::HandleSmoothScroll);

		connect (ScrollTimeline_,
				&QTimeLine::finished,
				[this] { emit isCurrentlyScrollingChanged (false); });
	}

	bool SmoothScroller::IsCurrentlyScrolling () const
	{
		return ScrollTimeline_->state () == QTimeLine::Running;
	}

	void SmoothScroller::SmoothCenterOn (qreal x, qreal y)
	{
		if (!XmlSettingsManager::Instance ().property ("SmoothScrolling").toBool ())
		{
			View_->centerOn (x, y);
			return;
		}

		const auto& current = View_->GetCurrentCenter ();
		XPath_ = qMakePair (current.x (), x);
		YPath_ = qMakePair (current.y (), y);

		if (ScrollTimeline_->state () != QTimeLine::NotRunning)
			ScrollTimeline_->stop ();

		emit isCurrentlyScrollingChanged (true);
		ScrollTimeline_->start ();
	}

	void SmoothScroller::HandleSmoothScroll (int frame)
	{
		const int endFrame = ScrollTimeline_->endFrame ();
		auto interp = [frame, endFrame] (const QPair<qreal, qreal>& pair)
				{ return pair.first + (pair.second - pair.first) * frame / endFrame; };
		View_->centerOn (interp (XPath_), interp (YPath_));
	}
}
}
