/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2010-2013  Oleg Linkin <MaledictusDeMagog@gmail.com>
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "chunkiodevice.h"

namespace LeechCraft
{
namespace NetStoreManager
{
namespace DBox
{
	ChunkIODevice::ChunkIODevice (const QString& path, QObject *parent)
	: QIODevice (parent)
	, ChunkSize_ (4 * 1024 * 1024)
	{
		File_.setFileName (path);
	}

	bool ChunkIODevice::atEnd () const
	{
		return File_.atEnd ();
	}

	bool ChunkIODevice::open (QIODevice::OpenMode mode)
	{
		return File_.open (mode);
	}

	void ChunkIODevice::close ()
	{
		File_.close ();
	}

	QByteArray ChunkIODevice::GetNextChunk ()
	{
		return File_.read (ChunkSize_);
	}

	qint64 ChunkIODevice::readData (char *data, qint64 maxlen)
	{
		return File_.read (data, maxlen);
	}

	qint64 ChunkIODevice::writeData (const char *data, qint64 len)
	{
		return File_.write (data, len);
	}
}
}
}
