/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "importwizard.h"
#include <QtDebug>
#include "importers/akregator/akregatorimporter.h"
#include "importers/firefox/firefoximporter.h"
#include "importers/kopete/kopeteimporter.h"
#include "importers/ktorrent/ktorrentimporter.h"
#include "importers/liferea/lifereaimporter.h"
#include "importers/psiplus/psiplusimporter.h"
#include "importers/vacuum/vacuumimporter.h"
#include "importers/opera/operaimporter.h"
#include "importers/jsonbookmarks/jsonbookmarksimporter.h"

namespace LeechCraft
{
namespace NewLife
{
	ImportWizard::ImportWizard (const ICoreProxy_ptr& proxy, QObject *plugin, QWidget *parent)
	: QWizard (parent)
	, Plugin_ (plugin)
	{
		Ui_.setupUi (this);

		using namespace Importers;
		std::apply ([&] (auto... tys) { Importers_ = { new std::remove_pointer_t<decltype (tys)> (proxy, this)... }; },
				std::tuple<
						AkregatorImporter*,
						FirefoxImporter*,
						OperaImporter*,
						JsonBookmarksImporter*,
						KTorrentImporter*,
						LifereaImporter*,
						KopeteImporter*,
						PsiPlusImporter*,
						VacuumImporter*
					> {});

		connect (this,
				&QDialog::accepted,
				this,
				&QObject::deleteLater,
				Qt::QueuedConnection);
		connect (this,
				&QDialog::rejected,
				this,
				&QObject::deleteLater,
				Qt::QueuedConnection);

		SetupImporters ();
	}

	QString ImportWizard::GetSelectedName () const
	{
		return Ui_.FirstPage_->GetSelectedName ();
	}

	QObject* ImportWizard::GetPlugin () const
	{
		return Plugin_;
	}

	void ImportWizard::SetupImporters ()
	{
		for (const auto ai : Importers_)
			Ui_.FirstPage_->SetupImporter (ai);
	}
}
}
