/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "icalgenerator.h"
#include <util/sll/prelude.h>
#include <interfaces/core/itagsmanager.h>
#include "core.h"

namespace LeechCraft
{
namespace Otlozhu
{
	ICalGenerator& ICalGenerator::operator<< (TodoItem_ptr item)
	{
		Items_ << item;
		return *this;
	}

	ICalGenerator& ICalGenerator::operator<< (QList<TodoItem_ptr> items)
	{
		Items_ += items;
		return *this;
	}

	namespace
	{
		QList<QByteArray> Serialize (TodoItem_ptr item)
		{
			QList<QByteArray> lines;
			lines << "BEGIN:VTODO";
			const QString& dateFt = "yyyyMMddTHHmmss";
			const auto& createdStr = item->GetCreatedDate ().toString (dateFt).toLatin1 ();
			lines << ("DTSTAMP:" + createdStr + "Z");
			lines << ("UID:" + item->GetID ().toLatin1 ());
			const auto& due = item->GetDueDate ();
			if (!due.isNull ())
				lines << ("DUE:" + due.toString (dateFt).toLatin1 ());
			lines << ("SUMMARY:" + item->GetTitle ().toUtf8 ().trimmed ());
			if (!item->GetComment ().isEmpty ())
				lines << ("COMMENT:" + item->GetComment ().toUtf8 ().trimmed ());
			lines << ("PERCENT-COMPLETE:" + QByteArray::number (item->GetPercentage ()));

			auto tm = Core::Instance ().GetProxy ()->GetTagsManager ();
			const auto& cats = Util::Map (item->GetTagIDs (), [tm] (const QString& id) { return tm->GetTag (id); });
			if (!cats.isEmpty ())
				lines << ("CATEGORIES:" + cats.join (",").toUtf8 ());

			lines << "END:VTODO";
			return lines;
		}
	}

	QByteArray ICalGenerator::operator() () const
	{
		QList<QByteArray> lines;
		lines << "BEGIN:VCALENDAR"
			<< "VERSION:2.0"
			<< "PRODID:-//LeechCraft//NONSGML Otlozhu//EN";
		lines += Util::ConcatMap (Items_, &Serialize);
		lines << "END:VCALENDAR";

		auto it = lines.begin ();
		while (it != lines.end ())
		{
			if (it->size () <= 75)
			{
				++it;
				continue;
			}

			const QByteArray line = *it;
			*it = line.left (75);
			it = lines.insert (it + 1, '\t' + line.mid (75));
		}

		QByteArray result;
		for (const auto& arr : lines)
			result += arr + "\r\n";
		return result;
	}
}
}
