/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "core.h"
#include <QTimer>
#include "storage.h"
#include "accountsmanager.h"
#include "operationsmanager.h"
#include "currenciesmanager.h"

namespace LeechCraft
{
namespace Poleemery
{
	Core::Core ()
	: Storage_ (new Storage)
	, AccsManager_ (new AccountsManager (Storage_))
	, OpsManager_ (new OperationsManager (Storage_))
	, CurrenciesManager_ (new CurrenciesManager ())
	{
		QTimer::singleShot (0,
				this,
				SLOT (postInit ()));
	}

	Core& Core::Instance ()
	{
		static Core c;
		return c;
	}

	void Core::SetCoreProxy (ICoreProxy_ptr proxy)
	{
		Proxy_ = proxy;
		CurrenciesManager_->Load ();
	}

	ICoreProxy_ptr Core::GetCoreProxy () const
	{
		return Proxy_;
	}

	Storage_ptr Core::GetStorage () const
	{
		return Storage_;
	}

	AccountsManager* Core::GetAccsManager () const
	{
		return AccsManager_;
	}

	OperationsManager* Core::GetOpsManager () const
	{
		return OpsManager_;
	}

	CurrenciesManager* Core::GetCurrenciesManager () const
	{
		return CurrenciesManager_;
	}

	void Core::postInit ()
	{
		OpsManager_->Load ();
	}
}
}
