/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#pragma once

#include <QString>
#include <QUrl>
#include <QDateTime>
#include <QPixmap>
#include <QMetaType>
#include <QMap>

namespace LeechCraft
{
namespace Poshuku
{
	/** Enumeration describing the part of menu that's being constructed
	 * inside QWebView's subclass' contextMenuEvent.
	 */
	enum WebViewCtxMenuStage
	{
		/// Just the beginning of menu construction.
		WVSStart,
		/// Stage related to clicking on a hyperlink finished.
		WVSAfterLink,
		/// Stage related to clicking on an image finished.
		WVSAfterImage,
		/// Stage related to clicking with having some selected text
		/// finished.
		WVSAfterSelectedText,
		/// The standard set of actions was embedded. This stage is just
		/// before executing the menu.
		WVSAfterFinish
	};

	struct HistoryItem
	{
		QString Title_;
		QDateTime DateTime_;
		QString URL_;
	};

	typedef QList<HistoryItem> history_items_t;

	struct ContextMenuInfo
	{
		bool IsContentEditable_;

		QString SelectedPageText_;

		QUrl LinkUrl_;
		QString LinkText_;

		QUrl ImageUrl_;
		QPixmap ImagePixmap_;
	};

	struct ElementData
	{
		QUrl PageURL_;
		QString FormID_;
		QString Name_;
		QString Type_;
		QString Value_;
	};

	inline bool operator== (const ElementData& left, const ElementData& right)
	{
		return left.PageURL_ == right.PageURL_ &&
			   left.FormID_ == right.FormID_ &&
			   left.Name_ == right.Name_ &&
			   left.Type_ == right.Type_ &&
			   left.Value_ == right.Value_;
	}

	inline bool operator< (const ElementData& left, const ElementData& right)
	{
		if (left.PageURL_ != right.PageURL_)
			return left.PageURL_ < right.PageURL_;

		if (left.FormID_ != right.FormID_)
			return left.FormID_ < right.FormID_;

		if (left.Name_ != right.Name_)
			return left.Name_ < right.Name_;

		if (left.Type_ != right.Type_)
			return left.Type_ < right.Type_;

		return left.Value_ < right.Value_;
	}

	using ElementsData_t = QList<ElementData>;

	/** Holds information about all the forms on a page.
	 *
	 * The key of the map is the name of the `input' element, whereas
	 * value is the ElementData structure with the information about
	 * that element.
	 */
	using PageFormsData_t = QMap<QString, ElementsData_t>;
}
}

Q_DECLARE_METATYPE (LeechCraft::Poshuku::ElementData)
Q_DECLARE_METATYPE (LeechCraft::Poshuku::ElementsData_t)
Q_DECLARE_METATYPE (LeechCraft::Poshuku::PageFormsData_t)
