/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2010-2011  Oleg Linkin
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#ifndef PLUGINS_POSHUKU_PLUGINS_ONLINEBOOKMARKS_PLUGINS_READITLATER_READITLATERSERVICE_H
#define PLUGINS_POSHUKU_PLUGINS_ONLINEBOOKMARKS_PLUGINS_READITLATER_READITLATERSERVICE_H

#include <QObject>
#include <interfaces/core/icoreproxy.h>
#include <interfaces/structures.h>
#include <interfaces/ibookmarksservice.h>

class QNetworkReply;

namespace LeechCraft
{
namespace Poshuku
{
namespace OnlineBookmarks
{
namespace ReadItLater
{

	class ReadItLaterApi;
	class ReadItLaterAccount;

	class ReadItLaterService : public QObject
							, public IBookmarksService
	{
		Q_OBJECT
		Q_INTERFACES (LeechCraft::Poshuku::OnlineBookmarks::IBookmarksService)
	public:
		enum OperationType
		{
			OTAuth,
			OTRegister,
			OTDownload,
			OTUpload
		};

		struct Request
		{
			OperationType Type_;
			QString Login_;
			QString Password_;
		};
	private:
		ICoreProxy_ptr CoreProxy_;
		std::shared_ptr<ReadItLaterApi> ReadItLaterApi_;
		QHash<QNetworkReply*, Request> Reply2Request_;
		QList<ReadItLaterAccount*> Accounts_;
		QHash<IAccount*, QByteArray> Account2ReplyContent_;
	public:
		explicit ReadItLaterService (ICoreProxy_ptr);

		void Prepare ();

		IBookmarksService::Features GetFeatures () const override;
		QObject* GetQObject () override;
		QString GetServiceName () const override;
		QIcon GetServiceIcon () const override;
		QWidget* GetAuthWidget () override;
		void CheckAuthData (const QVariantMap&) override;
		void RegisterAccount (const QVariantMap&) override;
		void UploadBookmarks (QObject*, const QVariantList&) override;
		void DownloadBookmarks (QObject*, const QDateTime& from = QDateTime ()) override;

		ReadItLaterAccount* GetAccountByName (const QString&);
	private:
		void SendRequest (const QString&, const QByteArray&, Request);
		void RestoreAccounts ();
	private slots:
		void getReplyFinished ();
		void readyReadReply ();

		void saveAccounts () const override;
		void removeAccount (QObject*) override;
	signals:
		void accountAdded (QObjectList) override;
		void gotBookmarks (QObject*, const QVariantList&) override;
		void bookmarksUploaded () override;
	};
}
}
}
}

#endif // PLUGINS_POSHUKU_PLUGINS_ONLINEBOOKMARKS_PLUGINS_READITLATER_READITLATERSERVICE_H
