/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#pragma once

#include <QObject>
#include <QNetworkAccessManager>
#include <interfaces/iinfo.h>
#include <interfaces/iplugin2.h>
#include <interfaces/ihavesettings.h>
#include <interfaces/ihavediaginfo.h>
#include <interfaces/core/ihookproxy.h>
#include <interfaces/poshuku/iwebviewprovider.h>
#include <interfaces/poshuku/iinterceptablerequests.h>

namespace LeechCraft
{
namespace Poshuku
{
class IProxyObject;

namespace WebKitView
{
	class WebPluginFactory;
	class InterceptAdaptor;
	class CustomWebView;

	class Plugin : public QObject
				 , public IInfo
				 , public IPlugin2
				 , public IHaveSettings
				 , public IHaveDiagInfo
				 , public IWebViewProvider
				 , public IInterceptableRequests
	{
		Q_OBJECT
		Q_INTERFACES (IInfo
				IPlugin2
				IHaveSettings
				IHaveDiagInfo
				LeechCraft::Poshuku::IWebViewProvider
				LeechCraft::Poshuku::IInterceptableRequests)

		LC_PLUGIN_METADATA ("org.LeechCraft.Poshuku.WebKitView")

		ICoreProxy_ptr Proxy_;
		IProxyObject *PoshukuProxy_ = nullptr;

		WebPluginFactory *WebPluginFactory_ = nullptr;

		Util::XmlSettingsDialog_ptr XSD_;

		std::shared_ptr<InterceptAdaptor> Interceptor_;
	public:
		void Init (ICoreProxy_ptr) override;
		void SecondInit () override;
		void Release () override;
		QByteArray GetUniqueID () const override;
		QString GetName () const override;
		QString GetInfo () const override;
		QIcon GetIcon () const override;

		QSet<QByteArray> GetPluginClasses () const override;

		Util::XmlSettingsDialog_ptr GetSettingsDialog () const override;

		QString GetDiagInfoString () const override;

		IWebView_ptr CreateWebView () override;
		QIcon GetIconForUrl (const QUrl&) const override;
		QIcon GetDefaultUrlIcon () const override;

		void AddInterceptor (const Interceptor_t&) override;
	private:
		void HandleView (CustomWebView*);
	private slots:
		void handleWebViewCreated (const std::shared_ptr<CustomWebView>&, bool);
	public slots:
		void hookNAMCreateRequest (LeechCraft::IHookProxy_ptr,
				QNetworkAccessManager*,
				QNetworkAccessManager::Operation*,
				QIODevice**);

		void initPlugin (QObject*);
	signals:
		void webViewCreated (const std::shared_ptr<IWebView>&, bool) override;
	};
}
}
}
