/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2011  Alexander Konovalov
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#pragma once

namespace LeechCraft
{
namespace SecMan
{
namespace SecureStorage
{
	const int HMACLength = 256 / 8;
	const int HashLength = 256 / 8;
	const int KeyLength = 256 / 8;
	const int IVLength = 128 / 8;
	const int RndLength = 4;

	namespace CipherTextFormatUtils
	{
		/**
		 * Compute buffer length for known data length.
		 * @param dataLength length of data.
		 * @return length of buffer.
		 */
		int BufferLengthFor (int dataLength);

		/**
		 * Compute data length from known buffer length.
		 * @param bufferlength of buffer.
		 * @return length of data.
		 */
		int DataLengthFor (int bufferLength);

		/**
		 * Compute decryption buffer length from known buffer length.
		 * Decryption buffer contains data and random block,
		 * and has length (Datalength + RND_LENGTH)
		 * @param bufferlength of buffer.
		 * @return length of buffer for decryption.
		 */
		int DecryptBufferLengthFor (int bufferLength);
	}

	/**
	 * Block of ciphertext in specific format.
	 *
	 * Format:
	 * IV + CIPHER(TEXT+RND) + HMAC(TEXT+RND)
	 */
	class CipherTextFormat
	{
		/** Pointer to buffer containing ciphertext. */
		unsigned char *Buffer_;
		/** Length of unencrypted data. */
		int DataLength_;

	public:
		CipherTextFormat (void *buffer, int dataLength);
		/** Pointer to initialization vector. */
		unsigned char* Iv () const;
		/** Pointer to encrypted data block. */
		unsigned char* Data () const;
		/** Pointer to encrypted random block. */
		unsigned char* Rnd () const;
		/** Pointer to HMAC. */
		unsigned char* Hmac () const;
		/** Pointer to beginning of buffer. */
		unsigned char* BufferBegin () const;
		/** Pointer to end of buffer. */
		unsigned char* BufferEnd () const;
		/** Length of data.*/
		int GetDataLength () const;
	};
}
}
}
