/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "attachmentsfetcher.h"
#include <QTemporaryDir>
#include <util/threads/futures.h>
#include <util/sll/visitor.h>
#include "account.h"

namespace LeechCraft
{
namespace Snails
{
	AttachmentsFetcher::AttachmentsFetcher (Account *acc,
			const QStringList& folder, const QByteArray& msgId, const QStringList& attNames)
	: Acc_ { acc }
	, Folder_ { folder }
	, MsgId_ { msgId }
	, AttQueue_ { attNames }
	{
		Promise_.reportStarted ();

		RotateQueue ();
	}

	QFuture<AttachmentsFetcher::Result_t> AttachmentsFetcher::GetFuture ()
	{
		return Promise_.future ();
	}

	void AttachmentsFetcher::RotateQueue ()
	{
		if (AttQueue_.isEmpty ())
		{
			Util::ReportFutureResult (Promise_, Result_t::Right ({ TempDir_, Paths_ }));
			return;
		}

		if (!TempDir_)
		{
			TempDir_ = std::make_shared<QTemporaryDir> ();
			if (!TempDir_->isValid ())
			{
				qWarning () << Q_FUNC_INFO
						<< "unable to create temporary directory";
				Util::ReportFutureResult (Promise_, Result_t::Left (TemporaryDirError {}));
				return;
			}
		}

		const auto& attName = AttQueue_.takeFirst ();
		const auto& filePath = QDir { TempDir_->path () }.filePath (attName);
		Util::Sequence (Acc_, Acc_->FetchAttachment (Folder_, MsgId_, attName, filePath)) >>
				Util::Visitor
				{
					[=] (Util::Void)
					{
						Paths_ << filePath;
						RotateQueue ();
					},
					[=] (auto e) { Util::ReportFutureResult (Promise_, Result_t::Left (e)); }
				};
}
}
}
