/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "core.h"
#include <QCoreApplication>
#include <QtDebug>

#ifdef Q_OS_WIN32
#include <vmime/platforms/windows/windowsHandler.hpp>
#else
#include <vmime/platforms/posix/posixHandler.hpp>
#endif

#include <util/sys/resourceloader.h>
#include <interfaces/core/ientitymanager.h>
#include "progressmanager.h"
#include "accountfoldermanager.h"
#include "composemessagetab.h"

namespace LeechCraft
{
namespace Snails
{
	Core::Core ()
	: ProgressManager_ { new ProgressManager { this } }
	, MsgView_ { new Util::ResourceLoader { "snails/msgview" } }
	{
#ifdef Q_OS_WIN32
		vmime::platform::setHandler<vmime::platforms::windows::windowsHandler> ();
#else
		vmime::platform::setHandler<vmime::platforms::posix::posixHandler> ();
#endif

		MsgView_->AddGlobalPrefix ();
		MsgView_->AddLocalPrefix ();

		qRegisterMetaType<size_t> ("size_t");
		qRegisterMetaType<AttDescr> ("LeechCraft::Snails::AttDescr");
		qRegisterMetaType<AttDescr> ("AttDescr");
		qRegisterMetaType<QList<QStringList>> ("QList<QStringList>");
		qRegisterMetaType<QList<QByteArray>> ("QList<QByteArray>");
		qRegisterMetaType<Folder> ("LeechCraft::Snails::Folder");
		qRegisterMetaType<QList<Folder>> ("QList<LeechCraft::Snails::Folder>");

		qRegisterMetaTypeStreamOperators<AttDescr> ();
		qRegisterMetaTypeStreamOperators<Folder> ();
		qRegisterMetaTypeStreamOperators<QList<Folder>> ();
	}

	Core& Core::Instance ()
	{
		static Core c;
		return c;
	}

	void Core::Release ()
	{
		MsgView_.reset ();
	}

	void Core::SetProxy (ICoreProxy_ptr proxy)
	{
		Proxy_ = proxy;
	}

	ICoreProxy_ptr Core::GetProxy () const
	{
		return Proxy_;
	}

	void Core::SendEntity (const Entity& e)
	{
		Proxy_->GetEntityManager ()->HandleEntity (e);
	}

	ProgressManager* Core::GetProgressManager () const
	{
		return ProgressManager_;
	}

	QString Core::GetMsgViewTemplate () const
	{
		auto dev = MsgView_->Load ("default/msgview.css");
		if (!dev || !dev->open (QIODevice::ReadOnly))
			return {};

		return QString::fromUtf8 (dev->readAll ());
	}
}
}
