/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "wkplugins.h"
#include <QIcon>
#include <xmlsettingsdialog/xmlsettingsdialog.h>
#include <util/util.h>
#include "staticplugin.h"
#include "notificationsext.h"
#include "spellcheckext.h"
#include "xmlsettingsmanager.h"

namespace LeechCraft
{
namespace WKPlugins
{
	void Plugin::Init (ICoreProxy_ptr proxy)
	{
		Util::InstallTranslator ("wkplugins");

		Proxy_ = proxy;

		XSD_ = std::make_shared<Util::XmlSettingsDialog> ();
		XSD_->RegisterObject (&XmlSettingsManager::Instance (), "wkpluginssettings.xml");

		StaticPlugin::SetImpl (this);
	}

	void Plugin::SecondInit ()
	{
	}

	QByteArray Plugin::GetUniqueID () const
	{
		return "org.LeechCraft.WKPlugins";
	}

	void Plugin::Release ()
	{
	}

	QString Plugin::GetName () const
	{
		return "WKPlugins";
	}

	QString Plugin::GetInfo () const
	{
		return tr ("Provides support for spellchecking and HTML5 notifications to WebKit.");
	}

	QIcon Plugin::GetIcon () const
	{
		return QIcon ();
	}

	Util::XmlSettingsDialog_ptr Plugin::GetSettingsDialog () const
	{
		return XSD_;
	}

	bool Plugin::supportsExtension (Extension ext) const
	{
		switch (ext)
		{
		case Extension::Notifications:
		case Extension::SpellChecker:
			return true;
		default:
			return false;
		}
	}

	QObject* Plugin::createExtension (Extension ext) const
	{
		QObject *extObj = nullptr;
		switch (ext)
		{
		case Extension::Notifications:
			extObj = new NotificationsExt { Proxy_ };
			break;
		case Extension::SpellChecker:
			extObj = new SpellcheckerExt { Proxy_ };
			break;
		default:
			break;
		}
		return extObj;
	}
}
}

LC_EXPORT_PLUGIN (leechcraft_wkplugins, LeechCraft::WKPlugins::Plugin);

Q_IMPORT_PLUGIN (StaticPlugin)
