/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#pragma once

#include <QObject>
#include <QMap>
#include <util/sll/eithercont.h>
#include "structures.h"

namespace LeechCraft
{
namespace XProxy
{
	class UrlListScript;
	class ScriptsManager;

	class ProxiesStorage : public QObject
	{
		const ScriptsManager * const ScriptsMgr_;

		QList<QPair<Proxy, QList<ReqTarget>>> Proxies_;
		QMap<Proxy, QList<UrlListScript*>> Scripts_;
	public:
		ProxiesStorage (const ScriptsManager*, QObject* = nullptr);

		QList<Proxy> GetKnownProxies () const;

		QList<Proxy> FindMatching (const QString& reqHost, int reqPort,
				const QString& proto = QString ()) const;

		void AddProxy (const Proxy&);
		void UpdateProxy (const Proxy&, const Proxy&);
		void RemoveProxy (const Proxy&);

		QList<ReqTarget> GetTargets (const Proxy&) const;
		void SetTargets (const Proxy&, const QList<ReqTarget>&);

		QList<UrlListScript*> GetScripts (const Proxy&) const;
		void SetScripts (const Proxy&, const QList<UrlListScript*>&);

		void Swap (int, int);

		void LoadSettings ();
		void SaveSettings () const;
	private:
		void EraseFromProxiesList (const Proxy&);

		template<typename R = void>
		R DoOnProxiesList (const Proxy&,
				const Util::EitherCont<R (), R (decltype (Proxies_.begin ()))>&);

		template<typename R = void>
		R DoOnProxiesList (const Proxy&,
				const Util::EitherCont<R (), R (decltype (Proxies_.constBegin ()))>&) const;
	};
}
}
