/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "monadplustest.h"

#include "monadtest.h"
#include <QtTest>
#include <monadplus.h>
#include <lazy.h>
#include <typelist.h>

QTEST_MAIN (LeechCraft::Util::MonadPlusTest)

namespace LeechCraft
{
namespace Util
{
	void MonadPlusTest::testBoostOptionalMplus ()
	{
		const boost::optional<int> val1 { 1 };
		const boost::optional<int> val2 { 2 };
		const auto nothing = Mzero<boost::optional<int>> ();

		const auto res1 = val1 + val2;
		const auto res2 = val1 + nothing;
		const auto res3 = nothing + val1;
		const auto res4 = nothing + nothing;

		QCOMPARE (res1, val1);
		QCOMPARE (res2, val1);
		QCOMPARE (res3, val1);
		QCOMPARE (res4, nothing);
	}

	void MonadPlusTest::testBoostOptionalMsum ()
	{
		const boost::optional<int> val1 { 1 };
		const boost::optional<int> val2 { 2 };
		const boost::optional<int> val3 { 3 };
		const auto nothing = Mzero<boost::optional<int>> ();

		const auto res1 = Msum ({ val1, val2, val3 });
		const auto res2 = Msum ({ val1, nothing });
		const auto res3 = Msum ({ nothing, val1 });
		const auto res4 = Msum ({ nothing, nothing });
		const auto res5 = Msum ({ nothing });

		QCOMPARE (res1, val1);
		QCOMPARE (res2, val1);
		QCOMPARE (res3, val1);
		QCOMPARE (res4, nothing);
		QCOMPARE (res5, nothing);
	}

	void MonadPlusTest::testLazyBoostOptionalMsum ()
	{
		const auto val1 = MakeLazy (boost::optional<int> { 1 });
		const auto val2 = MakeLazy (boost::optional<int> { 2 });
		const auto val3 = MakeLazy (boost::optional<int> { 3 });
		const auto nothing = MakeLazy (Mzero<boost::optional<int>> ());

		const auto res1 = Msum ({ val1, val2, val3 });
		const auto res2 = Msum ({ val1, nothing });
		const auto res3 = Msum ({ nothing, val1 });
		const auto res4 = Msum ({ nothing, nothing });
		const auto res5 = Msum ({ nothing });

		QCOMPARE (res1 (), val1 ());
		QCOMPARE (res2 (), val1 ());
		QCOMPARE (res3 (), val1 ());
		QCOMPARE (res4 (), nothing ());
		QCOMPARE (res5 (), nothing ());
	}
}
}
