/**********************************************************************
 * LeechCraft - modular cross-platform feature rich internet client.
 * Copyright (C) 2006-2014  Georg Rudoy
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer,
 * must be included in all copies of the Software, in whole or in part, and
 * all derivative works of the Software, unless such copies or derivative
 * works are solely in the form of machine-executable object code generated by
 * a source language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 **********************************************************************/

#include "slotclosuretest.h"
#include <QtTest>
#include "slotclosure.h"

QTEST_MAIN (LeechCraft::Util::SlotClosureTest)

namespace LeechCraft
{
namespace Util
{
	void DummyObject::EmitSignal ()
	{
		emit someSignal ();
	}

	void SlotClosureTest::testDeleteLater ()
	{
		DummyObject obj;

		bool hasRun = false;
		const auto closure = new SlotClosure<DeleteLaterPolicy>
		{
			[&hasRun]
			{
				hasRun = true;
			},
			&obj,
			SIGNAL (someSignal ()),
			nullptr
		};

		obj.EmitSignal ();

		const QPointer<QObject> closurePtr { closure };

		QCOMPARE (hasRun, true);
		QCOMPARE (closurePtr.isNull (), false);

		QCoreApplication::sendPostedEvents (nullptr, QEvent::DeferredDelete);

		QCOMPARE (closurePtr.isNull (), true);
	}

	void SlotClosureTest::testNoDelete ()
	{
		DummyObject obj;

		bool hasRun = false;
		const auto closure = new SlotClosure<NoDeletePolicy>
		{
			[&hasRun]
			{
				hasRun = true;
			},
			&obj,
			SIGNAL (someSignal ()),
			nullptr
		};

		obj.EmitSignal ();

		const QPointer<QObject> closurePtr { closure };

		QCOMPARE (hasRun, true);
		QCOMPARE (closurePtr.isNull (), false);

		QCoreApplication::sendPostedEvents (nullptr, QEvent::DeferredDelete);

		QCOMPARE (closurePtr.isNull (), false);
	}

	void SlotClosureTest::testChoiceDelete ()
	{
		DummyObject obj;

		bool hasRun = false;
		const auto closure = new SlotClosure<ChoiceDeletePolicy>
		{
			[&hasRun]
			{
				if (hasRun)
					return ChoiceDeletePolicy::Delete::Yes;

				hasRun = true;
				return ChoiceDeletePolicy::Delete::No;
			},
			&obj,
			SIGNAL (someSignal ()),
			nullptr
		};
		const QPointer<QObject> closurePtr { closure };

		obj.EmitSignal ();

		QCOMPARE (hasRun, true);
		QCOMPARE (closurePtr.isNull (), false);

		QCoreApplication::sendPostedEvents (nullptr, QEvent::DeferredDelete);

		QCOMPARE (closurePtr.isNull (), false);

		obj.EmitSignal ();
		QCoreApplication::sendPostedEvents (nullptr, QEvent::DeferredDelete);

		QCOMPARE (closurePtr.isNull (), true);
	}
}
}
